package service

import (
	"fmt"
	"iot_manager_service/app/device/dao"
	"iot_manager_service/util"
	"time"
)

// 中间件管理服务
var BridgeSensorService = new(bridgeSensorService)

type bridgeSensorService struct{}

func (s *bridgeSensorService) Get(id int) (*dao.BridgeSensor, *util.Errors) {
	// 创建查询实例
	device := &dao.BridgeSensor{
		ID: id,
	}
	err := device.GetDevice()
	if err != nil {
		return nil, util.FailResponse(err.Error(), nil)
	}
	return device, nil
}

func (s *bridgeSensorService) CreateOrUpdate(userId int64, tenantId int, req dao.BridgeSensor) *util.Errors {
	// 创建查询实例
	device := req
	device.TenantID = tenantId
	device.UpdateUser = userId
	device.UpdateTime = time.Now()
	if req.ID == 0 {
		device.CreateTime = time.Now()
		device.CreateUser = userId
		if device.IsExistedBySN() {
			fmt.Printf("Create GetDeviceID err \n")
			return util.ParamsInvalidResponse("编码不能重复，请重新填写！", nil)
		}
		if err := device.Create(); err != nil {
			fmt.Printf("Create err = %s \n", err.Error())
			return util.FailResponse(err.Error(), nil)
		}
		return util.SuccessResponse(util.Succeeded, nil)
	}
	if device.IsExistedByNameAndCode() {
		return util.ParamsInvalidResponse("编码不能重复，请重新填写！", nil)
	}

	if err := device.Update(); err != nil {
		fmt.Printf("Update err = %s \n", err.Error())
		return util.FailResponse(err.Error(), nil)
	}

	//todo operation record
	return util.SuccessResponse(util.Succeeded, nil)
}
func (s *bridgeSensorService) List(searchValue string, current, size int) ([]dao.BridgeSensor, *util.Errors) {
	device := dao.BridgeSensor{}
	if searchValue != "" {
		device.SN = searchValue
		device.Name = searchValue
	}

	offset := (current - 1) * size
	limit := size
	devices, err := device.GetDevices(offset, limit)
	if err != nil {
		return nil, util.FailResponse(err.Error(), nil)
	}
	return devices, nil
}
func (s *bridgeSensorService) Remove(userId int64, id int) *util.Errors {
	// 创建查询实例
	device := &dao.BridgeSensor{
		ID:         id,
		IsDeleted:  1,
		UpdateUser: userId,
		UpdateTime: time.Now(),
	}
	//todo
	// service.transformerService.CountRelation()

	//todo operation record
	err := device.Delete()
	if err != nil {
		return util.FailResponse(err.Error(), nil)
	}
	return nil
}

func (s *bridgeSensorService) GetList(tenantId int) ([]*dao.BridgeSensor, *util.Errors) {
	var devices []*dao.BridgeSensor
	err := util.Redis.Get(getTransformerListRedisKey(tenantId)).Scan(&devices)
	if err == nil {
		return devices, nil
	}

	device := &dao.BridgeSensor{
		TenantID:  tenantId,
		IsDeleted: 0,
	}
	devices, err = device.GetAllDevices()
	if err != nil {
		return nil, util.FailResponse(err.Error(), nil)
	}
	_ = util.Redis.Set(getTransformerListRedisKey(tenantId), devices, 0).Err()

	return devices, nil
}

func (s *bridgeSensorService) Enable(id, status int) *util.Errors {
	// 创建查询实例
	device := &dao.BridgeSensor{
		ID:       id,
		IsEnable: status,
	}

	err := device.UpdateEnable()
	if err != nil {
		return util.FailResponse(err.Error(), nil)
	}
	return nil
}
